<?php # -*- coding: utf-8 -*-

define("OAK_TSMARK_MAX", 1000);

/**
 *  @class Oak_Action
 *  Lo escrito como base dentro de esta clase será heredado y llevado a
 *  cada una de las clases módulo que se escriban a posteriori.
 *
 *  @author       Manuel Angel Rubio Jimenez
 *  @version      0.1
 *
 */
abstract class Oak_Action {

    protected $session; /*!< Es una instancia de la clase del mismo nombre. */
    protected $template; /*!< Es una instancia de la clase del mismo nombre. */

    protected $alert; /*!< Almacena el/los mensaje/s informativos, de error, de confirmación, etc. */
    public $clean; /*!< Indica si se tienen que borrar los argumentos y el post. */
    
    private $tsmark; /*! Almacena una marca de tiempo para no repetir POST. */

    /**
     *  Operación que se encarga de inicializar todos los atributos de la
     *  clase.
     *
     *  Asigna la sesión pasada por parámetro a la variable de clase y
     *  crea una instancia de Template.
     *
     *  @param session objeto Session abierto en Main.
     *  @param template nombre de la plantilla a usar.
     *  @param type_template tipo de plantilla a usar: XSLT, Smarty...
     */
    function __construct( $session, $template, $type_template = "XSLT" ) {
        global $lang, $version;
        $this->session =& $session;
        $tpl = "Oak_Template_" . $type_template;
        $this->template = new $tpl($template);
        $this->clean = true;

        $this->template->put('main/@name', 'Oak');
        $this->template->put('main/@version', $version);
        $this->template->put("@lang", $lang);
        $access =& $this->session->get("access");
        $realname = '';
        if (is_subclass_of($access, 'Oak_Access')) {
            $user = $access->get("user");
            if (!is_scalar($user)) {
                $realname = $user->get('realname');
            }
        }
        $this->template->put("main/@subtitle", $realname);
        $timestamp =& $this->session->get("timestamp");
        $timestamp["counter"] = ($timestamp["counter"] + 1) % OAK_TSMARK_MAX;
        $this->tsmark = time() . "_" . sprintf("%03d", $timestamp["counter"]);
        if (!isset($timestamp["tsmark"]) or !is_array($timestamp["tsmark"])) {
            $timestamp["tsmark"] = array();
        }
        $this->template->put("main/timestamp/@tsmark", $this->tsmark);
    }
    
    /**
     *  Función correctora de nombre de argumentos.
     *
     *  En caso de que la URL se encuentre dentro del XML,
     *  el parser puede cambiar los & por &amp;, resultando
     *  el envío de variables como "amp;var". Esta función
     *  elimina los "amp;".
     *
     *  @param args (referencia) array de argumentos a corregir.
     */
    private function corrige_args( &$args ) {
        global $_SERVER;
        if (isset($args["amp;act"])) {
            $tmp = $args;
            foreach ($tmp as $k => $v) {
                $p = split(";", $k);
                $args[$p[1]] = $v;
                unset($args[$k]);
            }
            $_SERVER['REQUEST_URI'] = str_replace('&amp;', '&', $_SERVER['REQUEST_URI']);
        }
    }


    /**
     *  Fija el flujo de ejecución en la ejecución de <tt>form</tt> y, en
     *  caso de existir datos de formulario, primero ejecutaría <tt>post</tt>.
     *
     *  @param args argumentos para la ejecución de la acción.
     *  @param post datos de formulario.
     */
    public function run( $args, $post ) {
        $this->corrige_args($args);
        $timestamp =& $this->session->get("timestamp");
        if (empty($timestamp)) {
            $timestamp = array ( "tsmark" => array () );
        }
        if (!is_array($timestamp["tsmark"]) or !isset($post["tsmark"]) or
            !in_array($post["tsmark"], $timestamp["tsmark"])) {
            if (isset($post["tsmark"])) {
                $timestamp["tsmark"][] = $post["tsmark"];
                if (count($timestamp["tsmark"]) >= OAK_TSMARK_MAX) {
                    array_shift($timestamp["tsmark"]);        
                }
            }
            if (is_array($post) and count($post) > 0) {
                if (!is_bool($rt = $this->post($args, $post))) {
                    return $rt;
                }
				if (!$rt) {
					return true;
				}
            }
        }
        $this->form($args, $post);
        return true;
    }


    /**
     *  Vacía, esta operación debe de ser implementada en las clases hijas y
     *  debe de contener información de presentación, formularios, tablas o
     *  similar.
     *
     *  @param args argumentos para la ejecución de la acción.
     *  @param post datos del formulario.
     */
    abstract function form( $args, $post );


    /**
     *  Vacía, esta operación debe de ser implementada en las clases hijas y
     *  debe de contener la ejecución en caso de que haya datos pasados por
     *  post.
     *
     *  @param args argumentos para la ejecución de la acción.
     *  @param post datos del formulario.
     */
    abstract function post( $args, $post );
}
