<?php # -*- coding: utf-8 -*-

/**
 *  @class Oak_Action_Login
 *
 *  Login es la acción por defecto, o la primera acción cuando se accede
 *  al entorno web. Se encargará de tomar los datos de acceso y dar paso
 *  a la siguiente acción.
 *
 *  @author       Manuel Angel Rubio Jimenez
 *  @version      0.2
 *
 */
class Oak_Action_Login extends Oak_Action {

    /**
     *  Presentación del formulario de Login.
     *
     *  @param args argumentos para la ejecución de form.
     *  @return verdadero si todo hay ido bien, falso sino, y cadena si
     *          hay que ejecutar otra acción.
     */
    function form( $args, $post ) {
        $access =& $this->session->get("access");
        if (!is_subclass_of($access, 'Oak_Access')) {
            $c = new Oak_Components_Login($this->template, 'Login', '', 'Oak');
            $c->setUser(_("Usuario"), isset($post["username"]) ? $post["username"] : "");
            $c->setPass(_("Clave"));
            $this->template->put("main/login/access/@name", _('Acceso'));
            $c->setSubmit(_("Entrar"));
            if ($this->alert) {
                $c->setErrorMsg($this->alert);
            }
        } else {
            if (is_subclass_of($access, 'Oak_Access')) {
				$user =& $access->get("user");
                if (!$user or is_subclass_of($user, 'Oak_User')) {
                    $realname = _("Desconocido");
                } else {
                    $realname = $access->get("user")->get('realname');
                }
            }
            $c = new Oak_Components_Empty($this->template, 'Oak', $realname, 'Oak');
            $this->menu = new Oak_Menu($access);
            $this->menu->setTemplate($this->template);
        }
        print $c->render();
        return true;
    }


    /**
     *  Comprobación de Login.
     *
     *  @param args argumentos para la ejecución de la acción.
     *  @param post datos del formulario.
     *  @return verdadero si todo hay ido bien, falso sino, y cadena si
     *          hay que ejecutar otra acción.
     */
    function post( $args, $post ) {
        global $default_access;
        $acc = (isset($post["access"])) ? $post["access"] : (
			isset($args["access"]) ? $args["access"] : $default_access
		);
        $access = new $acc();

        if (!isset($post["username"]) or !isset($post["password"]) or
            !isset($post["usertype"])) {
            $this->alert = _("Sesión expiró por inactividad.");
            return false;
        }

        $user = $post["username"];
        $pass = $post["password"];
        try {
            $result = $access->login($user, $pass);
            if ($result == true) {
                $this->session->set('access', $access);
                return true;
            } else {
                $this->alert = _("Clave o usuario no incorrectos.");
            }
        } catch (Exception $e) {
            $this->alert = $e->getMessage();
        }
        return true;
    }

}
