<?php # -*- coding: utf-8 -*-

/* Agregamos código de autocarga de clases */
spl_autoload_register('oak_autoload');

/* método para autocarga en profundidad de las clases necesarias */
function oak_autoload( $class_name ) {
    if (file_exists_included(str_replace('_', '/', $class_name) . ".php")) {
        require_once str_replace('_', '/', $class_name) . ".php";
    }
}

/**
 * Check if a file exists in the include path and if it does,
 * return the absolute path.
 *
 * @param string $filename Name of the file to look for.
 * @return string|false The absolute path if file exists, false if it does not.
 */
function file_exists_included($filename) {
    // Check for absolute path
    if (realpath($filename) == $filename) {
        return $filename;
    }

    // Otherwise, treat as relative path
    $paths = explode(PATH_SEPARATOR, get_include_path());
    foreach ($paths as $path) {
        if (substr($path, -1) == DIRECTORY_SEPARATOR) {
            $fullpath = $path.$filename;
        } else {
            $fullpath = $path . DIRECTORY_SEPARATOR . $filename;
        }
        if (file_exists($fullpath)) {
            return $fullpath;
        }
    }
    return false;
}
