<?php # -*- coding: utf-8 -*-

/**
 *  @class Oak_Components_Login
 *  Componentes de inicio de sesión para crear plantillas basadas en la clase
 *  Templates.
 *
 *  @author       Manuel Angel Rubio Jimenez
 *  @version      0.2
 *
 */
class Oak_Components_Login extends Oak_Components {

    /**
     *  Constructor que se encarga de inicializar la estructura básica
     *  de un tipo de plantilla <em>login</em>.
     *
     *  @param template la plantilla que se usará para la salida.
     *  @param title título que se presentará en la barra de tareas y la
     *               ventana del navegador.
     *  @param subtitle se presentará en la cabecera de la página.
     *  @param name nombre que aparecerá como logo.
     *  @param lang idioma que se empleará.
     */
    function __construct( &$template, $title = "", $subtitle = "", $name = "", $lang = "" ) {
        parent::__construct($template, $title, $subtitle, $name, $lang);
        $this->template->put("main/@type", "login");
    }


    /**
     *  Configura el elemento <em>user</em> en la plantilla.
     *
     *  @param name texto de etiqueta junto a la entrada del usuario.
     *  @param value valor dentro de la entrada del usuario.
     */
    function setUser( $name, $value = "" ) {
        $this->template->put("main/login/user/@name", $name);
        $this->template->put("main/login/user/@value", $value);
    }


    /**
     *  Configura el elemento <em>pass</em> en la plantilla.
     *
     *  @param name texto de etiqueta junto a la entrada de la clave.
     *  @param value valor dentro de la entrada de la clave.
     */
    function setPass( $name, $value = "" ) {
        $this->template->put("main/login/pass/@name", $name);
        $this->template->put("main/login/pass/@value", $value);
    }


    /**
     *  Configura el elemento <em>type</em> en la plantilla.
     *
     *  @param name texto de etiqueta junto a la entrada del tipo.
     *  @param value valor dentro de la entrada del tipo.
     */
    function setType( $name, $value = "" ) {
        $this->template->put("main/login/type/@name", $name);
        $this->template->put("main/login/type/@value", $value);
    }


    /**
     *  Configura el elemento <em>submit</em> en la plantilla.
     *
     *  @param name texto de etiqueta que aparecerá en el botón.
     */
    function setSubmit( $name ) {
        $this->template->put("main/login/submit/@name", $name);
    }


    /**
     *  Configura el elemento <em>error-msg</em> en la plantilla.
     *
     *  @param value texto que se presentará como mensaje de error.
     */
    function setErrorMsg( $value ) {
        $this->template->put("main/login/error-msg/@value", $value);
    }

}
