<?php # -*- coding: utf-8 -*-

/**
 *  @class Oak_Date
 *  Gestión de Fechas y Horas.
 *
 *  @author       Manuel Angel Rubio Jimenez
 *  @version      0.2
 *
 */
class Oak_Date {

    private $date;
    private $format;

    /**
     *  Constructor. Inicializa las variables.
     *
     *  @param fmt formato de la fecha para las salidas.
     */
    public function __construct( $fmt = "d-m-Y" ) {
        $this->format = $fmt;
        $this->date = time();
    }

    /**
     * Asigna la fecha y/o hora pasada como parámetro.
     *
     * @param string $date fecha y hora en forma de cadena de texto.
     */
    public function setDate( $date ) {
        if (preg_match('/^[0-9]+$/', $date)) {
            $this->date = $date;
            return true;
        }
        if (preg_match("/^([0-9]{4})-([0-9]{2})-([0-9]{2})/", $date, $args)) {
            list ($dummy, $year, $month, $day) = $args;
        } elseif (preg_match("/^([0-9]{2})/([0-9]{2})/([0-9]{4})/", $date, $args)) {
            list ($dummy, $day, $month, $year) = $args;
        }
        if (preg_match("/([0-9]{2}):([0-9]{2}):?([0-9]{2})?/", $date, $args)) {
            list ($dummy, $hour, $minute, $second) = $args;
        }
        $this->date = mktime($hour, $minute, $second, $month, $day, $year);
        return true;
    }

    /**
     * Toma la fecha guardada en formato Unix de segundos.
     *
     * @return long tiempo en formato Unix.
     */
    public function getDate() {
        return $this->date;
    }
    
    /**
     * Configura el formato de la fecha al usar la clase con
     * la función __toString.
     *
     * @param string $fmt formato reconocible por la función date de PHP.
     */
    public function setFormatDate( $fmt ) {
        $this->format = $fmt;
    }
    
    /**
     * Función especial para tomar la fecha en el formato deseado.
     *
     * @return string fecha formateada.
     */
    public function __toString() {
        return date($this->format, $this->date);
    }
}
