<?php # -*- coding: utf-8 -*-

/**
 *  @class Oak_Environment
 *  Contiene los datos del entorno al que se realiza la petición. Será
 *  el nombre del dominio, la plantilla a usar y datos relacionados a
 *  estos.
 *
 *  @author       Manuel Angel Rubio Jimenez
 *  @version      0.2
 *
 */
class Oak_Environment {

    private $domain = "";     /*!< el dominio o máquina (host) a través de la
                                   que se accede al entorno. */
    private $template = "";   /*!< la plantilla que se empleará para ese
                                   dominio o máquina. */
    private $type_template = ""; /*!< el tipo de plantilla que se empleara: XSLT, Smarty... */

	private static $singleton = null; /*!< instancia solo un environment. */

    /**
     *  Toma los datos de environment dado el dominio.
     *
     *  @param domain dominio del entorno que se quiere tomar.
     *  @return objeto de tipo Environment.
     */
    public static function &getEnv( $domain ) {
		global $environment;
		if (empty(self::$singleton)) {
        	self::$singleton = new Oak_Environment($domain);
		}
		$rt =& self::$singleton;
        if (isset($environment[$domain])) {
            $rt->set("domain", $domain);
            $rt->set("template", $environment[$domain]["template"]);
            $rt->set("type_template", $environment[$domain]["type_template"]);
        }
        return $rt;
    }

	/**
	 *  Constructor del entorno (environment). Se encarga de
	 *  establecer los valores por defecto en el entorno
	 *  recién creado.
	 *
	 *  @param string $domain dominio del entorno.
	 */
	public function __construct( $domain = '' ) {
		global $default_theme, $default_type_template;
		
		$this->domain = $domain;
		$this->template = $default_theme;
		$this->type_template = $default_type_template;
	}

    /**
     *  Toma el valor de un atributo del objeto de tipo User.
     *
     *  @param key nombre de un atributo.
     *  @return valor del atributo.
     */
    public function get( $key ) {
        return $this->$key;
    }


    /**
     *  Establece el valor de un atributo del objeto de tipo User.
     *
     *  @param key nombre del atributo.
     *  @param val valor a asignarle.
     */
    public function set( $key, $val ) {
        $this->$key = $val;
    }
}
