<?php # -*- coding: utf-8 -*-

// workaround: con is_subclass_of no se autocargan las clases.
include_once("Oak/User.php");

/**
 *  @class Oak_Main
 *  Es la clase principal, por la que se accederá siempre ante cada
 *  petición, carga la acción y la ejecuta.
 *
 *  @author       Manuel Angel Rubio Jimenez
 *  @version      0.2
 *
 */
class Oak_Main {

    private $forward = array (); /*!< Almacena siguiente acción solicitada, con atributos. */

	/**
	 *  Inicializa los elementos de Main.
	 */
	public function __construct() {
        // montamos el sistema de logs
        if (!isset($_ENV["OS"])) {
            openlog("oak-framework", LOG_PID | LOG_NDELAY, LOG_LOCAL0);
        }
	}

	/**
	 *  Libera los recursos adquiridos por la instancia de Main.
	 */
	public function __destruct() {
        if (!isset($_ENV["OS"])) {
            closelog();
        }
	}

    /**
     *  Esta operación es la principal de esta clase, cuando el programa
     *  es lanzado por el servidor web, la ejecución hace que se cree una
     *  instancia de esta clase y sea lanzada esta operación.
     *
     *  Se encargará de llevar a cabo toda la ejecución del entorno y
     *  finalizará tras cada petición.
     *
     *  @param action la acción a ejecutar.
     *  @param args array con el resto de parámetros pasados por GET.
     *  @param post array con los datos pasados por POST.
     *  @param domain dominio con el que se ha accedido a la página.
     */
    function run( $action, $args, $post, $domain ) {
        global $default_action, $default_access, $session_manager, $anonymous_actions;
        global $modules_dir, $oak_modules_dir;
        global $check_env, $default_theme, $default_type_template;
        global $_ENV;

        // creamos una sesión
        if ($session_manager == "") {
            $session = new Oak_Session_Local();
        } else {
            $session_manager = "Oak_Session_" . $session_manager;
            $session = new $session_manager();
        }
        
        // quitamos el evento, en caso de que lo haya
        if (is_array($action)) {
            list($action, $event) = explode("@", $action);
        }
        if (empty($event)) {
            $event = "run";
        }

        // tomamos el acceso, sino existe se cambia la acción a realizar
        $access =& $session->get("access");
        if (is_array($anonymous_actions) and in_array($action, $anonymous_actions)) {
            if (!$access or !is_subclass_of($access, 'Oak_Access')) {
                $access = new $default_access();
                if (!is_subclass_of($access, 'Oak_Access')) {
                    // FIXME: desarrollar una clase de excepción específica para este punto
					throw new Exception(_("clase de acceso no encontrada"));
                }
            }
        } elseif (!is_subclass_of($access, 'Oak_Access')) {
            $action = $default_action;
            $event = "run";
        } else {
            // se selecciona el idioma en caso de que haya usuario
			$user =& $access->get('user');
            if ($user and is_subclass_of($user, 'Oak_User')) {
                $this->setLang($user->get('lang'));
            }
        }

        // se toma el entorno, sino existe se creará uno por defecto
        $env =& $session->get("env");
        if ($check_env === false) {
            $env = new Oak_Environment($domain);
        } elseif (!($env instanceof Oak_Environment)) {
            $env =& Oak_Environment::getEnv($domain);
            $session->set("env", $env);
        }

        // se ejecuta la acción, se pueden ejecutar en línea hasta 10 acciones
        for ($loop = 10; $loop > 0; $loop--) {
            if ($action == $default_action or $access->checkAccess($action)) {
                $parts = explode("_", $action);
                $file = array_pop($parts);
                $dir = implode("/", $parts);
                if (file_exists("$modules_dir/$dir/$file.php")) {
                    include_once("$modules_dir/$dir/$file.php");
                    $action_ex = new $action($session, $env->get("template"), $env->get("type_template"));
                } elseif (file_exists("$modules_dir/$dir/$file.php")) {
                    include_once("$modules_dir/$dir/$file.php");
                    $action_ex = new $action($session, $env->get("template"), $env->get("type_template"));
                } elseif (file_exists(dirname(__FILE__) . "/Action/$file.php")) {
                    include_once(dirname(__FILE__) . "/Action/$file.php");
                    $action = "Oak_Action_$file";
                    $action_ex = new $action($session, $env->get("template"), $env->get("type_template"));
                } else {
                    $rt = $default_action;
                }
				$methods = get_class_methods($action);
                if (is_array($methods) and in_array($event, $methods)) {
                    $rt = $action_ex->$event($args, $post);
                } else {
					$error = "acción %s y evento %s no existentes.";
                    throw new Exception(sprintf(_($error), $action, $event));
                }

                // si el retorno de la acción no es boleano, es que
                // es otra acción a la que hay que saltar.
                if (!is_bool($rt)) {
                    $action = $rt;
                    if ($action_ex->clean) {
                        $args = array ();
                        $post = array ();
                    }
                } elseif (count($this->forward) >= 1) {
                    $data = array_shift($this->forward);
                    $action = $data["action"];
                    $post = $data["post"];
                    $args = $data["args"];
                } else {
                    break;
                }
            } else {
                $action = $default_action;
            }
        }
    }


    /**
     *  Agrega una acción para ejecutar como siguiente con datos
     *  de POST y ARGS.
     *
     *  @param action acción a realizar.
     *  @param post array con datos de post a pasarle a la acción.
     *  @param args array con datos de arts a pasarle a la acción.
     */
    function forward( $action, $post = array(), $args = array() ) {
        array_push($this->forward, array (
            "action" => $action,
            "post" => $post,
            "args" => $args
        ));
    }


    /**
     *  Realiza una redirección a otra petición de página, con los
     *  parámetros solicitados.
     *
     *  @param action acción a la que redireccionar.
     *  @param args argumentos que pasarle.
     */
    function redirect( $action = "", $args = array() ) {
        if ($action == '') {
            $text = "Location: index.php";
        } else {
            $data = array();
            foreach ($args as $k => $i) {
                $data[] = urlencode($k) . "=" . urlencode($i);
            }
            $text = "Location: index.php?action=$action";
            if (count($data) > 0) {
                $text .= "&" . join("&", $data);
            }
        }
        header($text);
    }


    /**
     *  Configura el idioma para el resto de la ejecución del sistema.
     *
     *  El idioma llega por un parámetro (lang), normalmente en formato
     *  de dos caracteres (ISO-639), y se estandarizará en formato de
     *  ISO-639, guión bajo, ISO-3166-1, punto y tabla de caracteres.
     *
     *  @param lang idioma en que se configurará el sistema.
     */
    private function setLang( $new_lang ) {
        global $lang, $locale_dir;
        $lang = $new_lang;
        switch ($lang) {
            case "en":
            case "en_US":
            case "en_GB":
                $lang = "en_GB.utf8";
                break;
            default:
                $lang = "es_ES.utf8";
                break;
        }
        setlocale(LC_ALL, $lang);
        bindtextdomain('messages', $locale_dir);
        bind_textdomain_codeset('messages', 'UTF-8');
        textdomain('messages');
    }


    /**
     *  Escribe un mensaje de log al sistema syslog.
     *
     *  Hace uso de syslog para enviar un mensaje de aviso interno,
     *  que el usuario no podrá ver.
     *
     *  @param type tipo de mensaje: LOG_INFO, LOG_ERR...
     *  @param msg mensaje a enviar.
     */
    public function log( $type, $msg ) {
        if (!isset($_ENV["OS"])) {
            syslog($type, $msg);
        }
    }
}
