<?php # -*- coding: utf-8 -*-

/**
 *  @class Oak_Session
 *  Almacenará la información de Sesión.
 *
 *  Esta clase es un contenedor de lo que se almacenará en la sesión
 *  del usuario. La sesión se puede definir la consecución de peticiones
 *  del usuario desde la petición inicial de login hasta la petición
 *  final de logout.
 *
 *  @author       Manuel Angel Rubio Jimenez
 *  @version      0.2
 *
 */
abstract class Oak_Session {
    protected $session = array (); /*!< contenedor de datos de la sesión. */

    /**
     *  Toma la variable solicitada del contenedor de sesión.
     *
     *  Comprueba primero que la variable solicitada exista, sino es así
     *  retorna NULL.
     *
     *  @param name nombre del valor a rescatar.
     *  @return valor almacenado en la sesión o NULL en caso de fallo.
     */
    public function &get( $name ) {
        return $this->session[$name];
    }


    /**
     *  Guarda la variable solicitada del contenedor de sesión.
     *
     *  @param name nombre con el que se almacenará el valor.
     *  @param value valor a almacenar.
     */
    public function set( $name, &$value ) {
        $this->session[$name] =& $value;
    }


    /**
     *  Elimina la variable solicitada del contenedor de sesión.
     *
     *  @param name nombre de la variable que se eliminará.
     */
    public function remove( $name ) {
        unset($this->session[$name]);
    }


    /**
     *  Lista las sesiones que hay activas.
     *
     *  @return un array con el nombre de las sesiones activas.
     */
    public static function getList() {
        return array();
    }


    /**
     *  Elimina la sessión. Si se especifica el parámetro <em>file</em>,
     *  elimina una sesión que exista, no la actual.
     *
     *  @param file se especifica si se desea eliminar una sesión existente.
     */
    abstract public function deleteSession( $file = null );

}
