<?php # -*- coding: utf-8 -*-

/**
 *  @class Oak_Session_Memcache
 *  Almacenará la información de Sesión en un sistema Memcache.
 *
 *  Esta clase es un contenedor de lo que se almacenará en la sesión
 *  del usuario. La sesión se puede definir la consecución de peticiones
 *  del usuario desde la petición inicial de login hasta la petición
 *  final de logout.
 *
 *  Esta clase usará el sistema Memcache para tener mayor redundancia
 *  y escalabilidad.
 *
 *  @author       Manuel Angel Rubio Jiménez
 *  @version      0.2
 *
 */
class Oak_Session_Memcache extends Oak_Session {
    private $memcache; /*!< manejador del sistema memcache. */

    private $session_id;   /*!< identificador de la sesión. */
    private $session_name; /*!< nombre de la sesión. */
    private $session_die;  /*!< indica si la sesión se ha eliminado. */

    /**
     *  Operación que se encarga de abrir la sesión y realimentar el
     *  contenedor con los datos de la misma.
     */
    public function __construct( $session_name = "" ) {
        global $msession_host, $msession_port, $_COOKIE;
        if ($session_name == "") {
            $session_name = ini_get('session.name');
        }
        $this->memcache = new Memcache;
        $this->memcache->connect($msession_host, $msession_port);
        if (isset($_COOKIE[$session_name])) {
            $session_id = $_COOKIE[$session_name];
            $session_txt = $this->memcache->get($session_id);
            $this->session = unserialize($session_txt);
        } else {
            $session_id = md5(time() . rand());
            setcookie($session_name, $session_id);
            $this->session = array ();
        }

        $this->session_id = $session_id;
        $this->session_name = $session_name;
        $this->session_die = false;
    }


    /**
     *  Operación que se encarga de almacenar los datos de la memoria
     *  en el contenedor externo.
     */
    public function __destruct() {
        global $msession_expire;
        if (!$this->session_die) {
            $session_txt = serialize($this->session);
            $this->memcache->set($this->session_id, $session_txt, false, $msession_expire);
        }
        $this->memcache->close();
    }


    /**
     *  Lista las sesiones que hay activas.
     *
     *  @return un array con el nombre de las sesiones activas.
     */
    public static function getList() {
        // TODO
        return array ();
    }


    /**
     *  Elimina la sessión.
     *
     *  @param file se especifica si se desea eliminar una sesión existente.
     */
    public function deleteSession( $id = null ) {
        if (empty($id) and !empty($this->session_id)) {
            $id = $this->session_id;
		}
		if (!empty($id)) {
	        $this->memcache->set($session_id, '', false, 1);
	        $this->session = array ();
	        $this->session_die = true;
		}
    }

}
