<?php # -*- coding: utf-8 -*-

/**
 *  @class Oak_Template_Smarty
 *  Esta clase es una abstracción para Oak, para facilitar el uso de
 *  Smartys dentro del sistema.
 *  
 *  @author       Manuel Angel Rubio Jimenez
 *  @version      0.2
 *
 */
class Oak_Template_Smarty extends Oak_Template {

	private $tpl; //!< plantilla de Smarty
	private $smarty; //!< objeto de tipo Smarty

	/**
	 *  Constructor. Se encarga de crear el objeto Smarty.
	 */
	public function __construct( $template = null ) {
	    global $themes_dir, $smarty_tmpdir;
		parent::__construct($template);
		$this->smarty = new Smarty();
		$this->smarty->template_dir = $themes_dir;
		$this->smarty->compile_dir = $smarty_tmpdir;
		$this->tpl = "login";
		$this->xml_flavor = false;
	}

    /**
     *  Toma información desde la plantilla.
     *
     *  @param  path ruta para rescatar el valor solicitado.
     *  @return el valor solicitado, ya sea un <em>array</em> o una
     *          <em>string</em>.
     */
    public function get( $path ) {
        if ($path == "/main/@type" or $path == "main/@type") {
            return $this->tpl;
        }
        // reemplazamos caracteres no válidos
        $path = str_replace(
            array( "-" ),
            "_",
            $path
        );
		return parent::get($path);
	}


    /**
     *  Establece un valor para la plantilla.
     *
     *  @param path ruta para establecer el valor.
     *  @param value valor que se establecerá para la variable.
     */
    public function put( $path, $value ) {
        if ($path == "/main/@type" or $path == "main/@type") {
            $this->tpl = $value;
        } else {
            // reemplazamos caracteres no válidos
            $path = str_replace(
                array( "-" ),
                "_",
                $path
            );
            parent::put($path, $value);
        }
	}


    /**
     *  Convierte los valores dados en la salida deseada.
     *
	 *  @param test no tiene efecto en Smarty.
     *  @return texto HTML, generalmente.
     */
    public function render( $test = false ) {
        // FIXME: deben poder retornarse los datos. 
        if (!$test) {
            $tpl = $this->template . "/" . $this->tpl . ".tpl";
            $this->smarty->assign("MAIN", $this->data);
            $this->smarty->display($tpl);
        } else {
            print "Plantilla: " . $tpl . "<br />\n";
            print "Contenido: <br />\n<pre>";
            print_r($this->data);
            print "</pre>";
        }
	}

}
