<?php # -*- coding: utf-8 -*-

// necesario para los "define" necesarios.
include_once("XML/Serializer.php");

/**
 *  @class Oak_Template_XSLT
 *  Esta clase ha sido creada para que la salida hacia HTML, con el uso de
 *  XSLT, sea lo más MVC posible.
 *
 *  La clase se encarga de recrear un fichero XML dentro de una variable
 *  interna (<tt>data</tt>), a través de los métodos <tt>put</tt> y
 *  <tt>get</tt>. Cuando es invocado <tt>render</tt>, ese <em>array</em> es
 *  convertido a XML y fusionado con el XSLT elegido para terminar en un
 *  formato HTML presentable hacia el navegador web.
 *
 *  @author       Manuel Angel Rubio Jimenez
 *  @version      0.2
 *
 */
class Oak_Template_XSLT extends Oak_Template {

    /**
     *  El constructor se encarga de configurar la plantilla, inicialmente.
     *
     *  La única misión del constructor que se ha designado en estos momentos,
     *  es inicializar la variable de plantilla a un valor por defecto (joomla)
     *  o al valor pasado como parámetro.
     *
     *  @param template especifica la plantilla XSLT a usar.
     */
    function __construct( $template = "joomla" ) {
		parent::__construct($template);
        $this->xml_flavor = true;
    }


    /**
     *  Convierte un <em>array</em> a XML.
     *
     *  Este método privado se encarga de la conversión del árbol a XML.
     *
     *  @param data el árbol a convertir a XML.
     *  @return texto en XML.
     */
    private function Array2XML( $data ) {
        $options = array(
            "addDecl"            => true,
            "encoding"           => "utf-8",
            "mode"               => XML_SERIALIZER_MODE_SIMPLEXML,
            "indent"             => "    ",
            "linebreak"          => "\n",
            "scalarAsAttributes" => true,
            "attributesArray"    => '_a',
            "contentName"        => '_c',
            "rootName"           => 'main'
        );

        $serializer = new XML_Serializer($options);
        $serializer->serialize($this->parseXML($data));
        $doc = $serializer->getSerializedData();

        return $doc;
    }


    /**
     *  Método tomado de:
     *  http://es.php.net/manual/es/function.get-html-translation-table.php
     *  con modificaciones propias.
     *
     *  Convierte los acentos, y símbolos de <, >, etc, en su equivalente
     *  válido para XML.
     *
     *  @param s cadena a convertir.
     *  @return cadena convertida a formato válido XML.
     */
    private function xmlEntities( $s ) {
        //build first an assoc. array with the entities we want to match
        $table1 = get_html_translation_table(HTML_ENTITIES, ENT_QUOTES);

        //now build another assoc. array with the entities we want to replace (numeric entities)
        foreach ($table1 as $k => $v) {
            $table1[$k] = "/$v/";
            $c = htmlentities( $k, ENT_COMPAT, 'iso-8859-1');
            $table2[$c] = "&#".ord($k).";";
        }
        $table1["€"] = "/&euro;/";
        $table2["&euro;"] = "&#8364;";

        //now perform a replacement using preg_replace
        //each matched value in array 1 will be replaced with the corresponding value in array 2
        if (!is_array($s) and !is_object($s)) {
            // mantenemos esta línea para codificar simbolos que producen errores
            // en el parser de XML... pero con esta característica, las tildes se
            $s = htmlentities( $s, ENT_COMPAT, 'utf-8');
            $s = preg_replace($table1, $table2, $s);
        } else {
            $s = is_array($s) ? "(Error): Array" : "(Error): Object";
        }
        return $s;
    }


    /**
     *  Método privado de uso interno para convertir cadenas en textos XML válidos.
     *
     *  Este texto usa la función <tt>htmlentities</tt> para convertir los carácteres del texto
     *  tales como <, >, ', " y &, a sus equivalentes símbolos XML, recorriendo todo el
     *  árbol para ello.
     *
     *  @param data el array para realizar el parseo.
     *  @return el array con los datos parseados.
     */
    private function parseXML( $data ) {
        if (is_array($data)) {
            foreach ($data as $k => $v) {
                if (is_array($v)) {
                    $data[$k] = $this->parseXML($data[$k]);
                } else {
                    $data[$k] = $this->xmlEntities($data[$k]);
                }
            }
        } else {
            $data = $this->xmlEntities($data);
        }
        return $data;
    }


    /**
     *  Convierte el árbol en HTML para salida hacia el navegador web.
     *
     *  Este método se encarga de la transformación del árbol en texto XML,
     *  procesar el texto XML con la información XSLT y devolver un texto
     *  HTML.
     *
     *  @return texto HTML.
     */
    public function render( $test = false ) {
        global $themes_dir;
        // cargamos el fichero XML (convirtiendo el array en XML primero)
        $xml = DOMDocument::loadXML($this->Array2XML($this->data));
        if (file_exists("$themes_dir/{$this->template}/xslt/main.xslt")) {
            $xsl = DOMDocument::load("$themes_dir/{$this->template}/xslt/main.xslt");
        } else {
            /** FIXME: hay que arreglar esto de alguna forma. */
            print "<b>ERROR:</b> Plantilla no disponible ($themes_dir/{$this->template}/xslt/main.xslt)."; die;
        }

        if (!$test) {
            // creamos el procesador de XSLT
            $processor = new XSLTProcessor();

            // realizamos la transformación y retornamos el HTML
            $processor->importStyleSheet($xsl);
            return $processor->transformToXML($xml);
        }
        return $xml->saveXML();
    }
}
