<?php # -*- coding: utf-8 -*-

/**
 *  @class Oak_User
 *  Contendrá todos los datos de los usuarios, así como sus funcionalidades
 *  asociadas.
 *
 *  @author       Manuel Angel Rubio Jimenez
 *  @version      0.2
 *
 */
abstract class Oak_User {
    protected $name = "";       /*!< El nombre que se le otorga a la entidad. */
    protected $pass = "";       /*!< la clave del usuario. */
    protected $realname = "";   /*!< nombre y apellidos del usuario. */
    protected $email = "";      /*!< dirección de correo electrónico del usuario. */
    protected $lang = "";       /*!< idioma que usará el usuario. */


    /**
     *  Carga un usuario.
     *
     *  @param user_name nombre del usuario a cargar.
     *  @return objeto del usuario cargado o PEAR_Error con el error generado.
     */
    abstract public static function getUser( $user_name );


    /**
     *  Lista los usuarios.
     *
     *  @param ini desplazamiento a partir del que se empiezan a tomar datos.
     *  @param size número de tuplas que se obtienen.
     *  @return una tabla con el listado solicitado. Cada tupla es de tipo
     *          hash, mientras que las filas son arrays simples con índices
     *          numéricos.
     */
    abstract public function listUsers( $ini = "", $size = "" );


    /**
     *  Toma el valor de un atributo del objeto de tipo User.
     *
     *  @param key nombre de un atributo.
     *  @return valor del atributo.
     */
    public function get( $key ) {
        return $this->$key;
    }


    /**
     *  Establece el valor de un atributo del objeto de tipo User.
     *
     *  @param key nombre del atributo.
     *  @param val valor a asignarle.
     */
    public function set( $key, $val ) {
        $this->$key = $val;
    }
}
